import FreeCAD as App
import Part, Sketcher 
import Draft #Draft recomputes!

in_rad = 3.0
out_rad = 5.0
thickness = 5.0
R = thickness-0.5+in_rad
o = (in_rad+out_rad)/2.0
fillet_rad = 2.0
ADoc = App.ActiveDocument
aDoc = App.activeDocument()

vec = App.Vector
skstr = "Sketcher::SketchObject" #default = xy-plane
pdstr = "PartDesign::Pad"

def make_link(al1,a1,d1):
  global in_rad,out_rad,thickness,R,o,fillet_rad, ADoc, aDoc
  
  #U-connector
  skc = aDoc.addObject(skstr, "Sketchc")
  skc.addGeometry(Part.Arc(vec(-o,R,0),vec(-o-R,0,0),vec(-o,-R,0)))
  skc.addGeometry(Part.Arc(vec(-o,in_rad,0),vec(-o-in_rad,0,0),vec(-o,-in_rad,0)))
  skc.addGeometry(Part.Line(vec(-o,R,0),vec(-o,in_rad,0)))
  skc.addGeometry(Part.Line(vec(-o,-R,0),vec(-o,-in_rad,0)))
  skc.Placement = App.Placement(vec(0,0,0),App.Rotation(vec(1,0,0),90)) #xz-plane

  #end-point annuli
  sk1 = aDoc.addObject(skstr, "Sketch21")
  sk1.addGeometry(Part.Circle(vec(0,0,0),vec(0,0,1),out_rad))
  sk1.addGeometry(Part.Circle(vec(0,0,0),vec(0,0,1),in_rad))
  sk1.Placement=App.Placement(vec(0,0,thickness/2.0),App.Rotation(vec(0,0,1),0))
  #mirror
  sk2 = aDoc.addObject(skstr, "Sketch22")
  sk2.addGeometry(Part.Circle(vec(0,0,0),vec(0,0,1),out_rad))
  sk2.addGeometry(Part.Circle(vec(0,0,0),vec(0,0,1),in_rad))
  sk2.Placement=App.Placement(vec(0,0,-thickness/2.0),App.Rotation(vec(0,0,1),0))
  ADoc.recompute()
  Draft.rotate([skc,sk1,sk2],al1,vec(0,0,0),vec(1,0,0),False)
  Draft.move([skc,sk1,sk2],vec(a1,0,d1),False)

  #extrusions
  pd1 = aDoc.addObject(pdstr,"Pad21")
  pd1.Sketch = sk1
  pd1.Length = thickness
  pd2 = aDoc.addObject(pdstr,"Pad22")
  pd2.Sketch = sk2
  pd2.Reversed = 1
  pd2.Length = thickness
  pdc = aDoc.addObject(pdstr,"Padc")
  pdc.Sketch = skc
  pdc.Length = thickness
  pdc.Midplane = 1

  sk1.ViewObject.Visibility = False; sk2.ViewObject.Visibility = False
  skc.ViewObject.Visibility = False; 
  skc,sk1,sk2 = None, None, None

  #sweep-base extender
  skb = aDoc.addObject(skstr, "Sketchb")
  skb.addGeometry(Part.Circle(vec(0,0,0),vec(0,0,1),thickness/2.0))
  skb.Placement = App.Placement(vec(o,0,0),App.Rotation(vec(0,1,0),90)) #yz-plane
  #sweep-spline extender
  l = a1-2*o-(R+in_rad)/2
  sks = aDoc.addObject(skstr, "Sketchs")
  if abs(d1) > 0.1:
    if abs(a1)>30: #snake with 3 "linear pieces"
      sks.addGeometry(Part.Line(vec(o,0,0),vec(o+l/2,0,0)))
      sks.addGeometry(Part.Line(vec(o+l/2,0,0),vec(o+l/2,d1,0)))
      sks.addConstraint(Sketcher.Constraint("Coincident",0,2,1,1))
      sks.addGeometry(Part.Line(vec(o+l/2,d1,0),vec(o+l,d1,0)))
      sks.addConstraint(Sketcher.Constraint("Coincident",1,2,2,1))
      sks.fillet(0,2,fillet_rad)
      sks.fillet(1,2,fillet_rad)
    else: #two small to fillet, we create a snake with five "linear pieces"
      sks.addGeometry(Part.Line(vec(o,0,0),vec(o+20,0,0)))
      sks.addGeometry(Part.Line(vec(o+20,0,0),vec(o+20,d1/2,0)))
      sks.addGeometry(Part.Line(vec(o+20,d1/2,0),vec(o-20,d1/2,0)))
      sks.addGeometry(Part.Line(vec(o-20,d1/2,0),vec(o-20,d1,0)))
      sks.addGeometry(Part.Line(vec(o-20,d1,0),vec(o+l,d1,0)))
      for i in xrange(4): 
        sks.addConstraint(Sketcher.Constraint("Coincident",i,2,i+1,1))
      for i in xrange(4):
        sks.fillet(i,2,fillet_rad)
  else:
    sks.addGeometry(Part.Line(vec(o,0,0),vec(o+l,d1,0)))  
  sks.Placement = App.Placement(vec(0,0,0),App.Rotation(vec(1,0,0),90)) #xz-plane
  #extender
  sw = aDoc.addObject("Part::Sweep","Sweep")
  sw.Sections = [skb]
  sw.Spine = (sks, sks.Shape.Edges)
  sw.Solid = True
  sks,skb = None,None

  #base annulus
  sk = aDoc.addObject(skstr,"Sketch1")
  sk.addGeometry(Part.Circle(vec(0,0,0),vec(0,0,1),out_rad))
  sk.addGeometry(Part.Circle(vec(0,0,0),vec(0,0,1),in_rad))
  pd = aDoc.addObject(pdstr,"Pad1")
  pd.Sketch = sk
  pd.Length = thickness
  pd.Midplane = 1
  sk.ViewObject.Visibility = False

  #fus = aDoc.addObject("Part::MultiFuse","Fusion")
  #fus.Shapes = [pd,pd1,pd2,pdc,pde]

  ADoc.recompute()
