import rhinoscriptsyntax as rs

in_rad = 3.0
out_rad = 5.0
thickness = 5.0
R = thickness-0.5+in_rad
o = (in_rad+out_rad)/2.0
fillet_rad = 2.0

def make_link(al1,a1,d1):
  global in_rad,out_rad,thickness,R,o,fillet_rad
  
  #U-end
  pln = rs.PlaneFromNormal((0,0,thickness/2.0+0.5),(0,0,1))
  crv1 = rs.AddCircle(pln, out_rad)
  crv2 = rs.AddCircle(pln, in_rad)
  rs.SelectObjects([crv1,crv2])
  rs.Command("ExtrudeCrv _Solid=_Yes _BothSides=_No " + str(thickness-0.5))
  id1 = rs.LastObject()
  rs.CurrentView("Front")
  id2 = rs.MirrorObject(id1,(0,0,0),(1,0,0),True)
  pln = rs.WorldZXPlane()
  pln = rs.RotatePlane(pln,-180,[0,1,0])
  pln = rs.MovePlane(pln,[-o,0,0])
  crv1=rs.AddArc(pln,R,180)
  crv2=rs.AddArc(pln,in_rad,180)
  ln1 = rs.AddLine([-o,0,in_rad],[-o,0,R])
  ln2 = rs.AddLine([-o,0,-in_rad],[-o,0,-R])
  rs.SelectObjects([crv1,crv2,ln1,ln2])
  rs.Command("ExtrudeCrv _Solid=_Yes _BothSides=_Yes " + str(thickness/2.0))
  id3 = rs.LastCreatedObjects()
  #apply DH to U-end
  rs.RotateObjects([id1,id2,id3],[0,0,0],al1,[1,0,0])
  rs.MoveObjects([id1,id2,id3],[a1,0,d1])

  #O-end
  crv1 = rs.AddCircle(rs.WorldXYPlane(),out_rad)
  crv2 = rs.AddCircle(rs.WorldXYPlane(),in_rad)
  rs.SelectObjects([crv1,crv2])
  rs.Command("ExtrudeCrv _Solid=_Yes _BothSides=_Yes " + str(thickness/2.0))
  id4 = rs.LastCreatedObjects()

  #pipe
  l = a1-2*o-(R+in_rad)/2.0
  if abs(d1) > 0.1:
    if abs(a1)>30:
      crv1 = rs.AddPolyline([[o,0,0],[o+l/2,0,0],[o+l/2,0,d1],[o+l,0,d1]])
    else: 
      crv1 = rs.AddPolyline([[o,0,0],[o+20,0,0],[o+20,0,d1/2.0],\
       [o-20,0,d1/2.0],[o-20,0,d1],[o+l,0,d1]])
    rs.SelectObject(crv1)
    rs.Command("FilletCorners "+str(fillet_rad))
  else:
    crv1 = rs.AddLine([o,0,0],[o+l,0,d1])
  id5=rs.AddPipe(crv1,0,thickness/2.0,cap=1)
  rs.DeleteObject(crv1)
  rs.SelectObject(id5)

  #whole link
  rs.BooleanUnion([id1,id2,id3,id4,id5])
